--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

Game = class("Game")
Game:include(Stateful)

-- Init the game
function Game:initialize(soft)
	-- Setup variables
	if soft == nil then
		self.paused = false			-- If the game is paused
		self.options = false		-- Show the options
		self.currentLevel = nil		-- Current level
		self.player = nil			-- Player object
		self.instructions = {}		-- Player instructions
		self.guards = {}			-- Guards
		self.moves = 0				-- Moves tried
		self.tries = 1				-- Attempts tried
		self.damage = 0				-- Damage timer
		self.scale = 0				-- Scale of the game's graphics relative to the screen size
		self.aspectratio = 1		-- Aspect ratio for the game's graphics
		self.clouds = {				-- Location of the clouds
			x = -100,
			y = 100
		}
		self.levels = nil	-- Current levels
	end

	self.finishedLevel = false
	self.scale = config.screenwidth / love.graphics.getWidth()
	self.aspectratio = love.graphics.getWidth() / love.graphics.getHeight()

	-- Start the loading
	if soft == nil then
		self:gotoState("Loading")
	end
end

-- Play the game
function Game:play(level)
	-- Get the current level
	if level == nil then
		level = lib.loadLevel(self.currentLevel)
	else
		self.currentLevel = level
		level = lib.loadLevel(level)
	end

	-- Determine gridsize
	local gridsize = config.gridsize
	local w = (love.graphics.getWidth() * self.scale) / (table.getn(level.map) + 2)
	local h = (love.graphics.getHeight() * self.scale) / (table.getn(level.map[1]) + 2)
	if w < gridsize then gridsize = w end
	if h < gridsize then gridsize = h end

	self.gridsize = gridsize

	
	-- Current level
	self.level = LevelEntity:new(level)
	
	-- Reset moves
	self.moves = 0

	-- Set as not finished
	self.finishedLevel = false

	-- Is game paused?
	self.paused = false
	
	-- Player
	self.player = PlayerEntity:new(level.player.x, level.player.y)
	
	-- List of instructions for the player
	self.instructions = {}
	
	-- The guards patrolling around
	self.guards = {}
	for i=1,table.getn(level.guards) do
		local guard = GuardEntity:new(level.guards[i].x, level.guards[i].y)
		guard.direction = level.guards[i].direction
		table.insert(self.guards, guard)
	end
end

-- Reset cloud animation
function Game:resetClouds(x)
	self.clouds.x = x
	local function resetCloudsCallback()
		self:resetClouds(-media.images.clouds:getWidth())
	end
	tween(config.screenwidth, self.clouds, { x = config.screenwidth }, 'linear', resetCloudsCallback)
end

-- End the current level
function Game:endLevel()
	-- Get next level
	local nextLevel = nil
	for i,v in pairs(self.levels) do
		if self.currentLevel == v and i ~= table.getn(self.levels) then
			nextLevel = self.levels[i+1]
		end
	end

	-- Save results
	if not config.score then config.score = {} end
	if not config.score[self.currentLevel] or config.score[self.currentLevel] > self.moves then
		config.score[self.currentLevel] = self.moves
		lib.saveObject("config.lua", config)
	end


	-- Setup scroeboard
	local movesLabel = menu:get("moves")
	movesLabel:setText(self.moves)
	local triesLabel = menu:get("tries")
	triesLabel:setText(self.tries)
	local nextButton = menu:get("next-level")
	if not nextLevel then
		nextButton:setImages({
			default = media.images.button_back_normal,
			hover = media.images.button_back_hover,
			press = nil
		})
	else
		nextButton:setImages({
			default = media.images.button_next_normal,
			hover = media.images.button_next_hover,
			press = nil
		})
	end
	media.menus.scoreboard:refresh()

	-- Show
	media.menus.scoreboard:show()
	self.finishedLevel = true
end

-- Go to the next level
function Game:nextLevel()
	-- Get next level
	for i,v in pairs(self.levels) do
		if self.currentLevel == v and i ~= table.getn(self.levels) then
			self.tries = 1
			self:play(self.levels[i+1])
			return
		end
	end

	self:popState("Game")
end

-- Restart level
function Game:restartLevel(reset)
	if not reset then
		self.tries = self.tries + 1
	else
		self.tries = 1
	end
	self:play()
end

-- Quit the game
function Game:quit()
	love.event.push("q")
end

-- Draw the state
function Game:draw() end

-- Update the state
function Game:update(dt) end

-- Handle input
function Game:mousepressed(x, y, button) end
function Game:mousereleased(x, y, button) end
function Game:keypressed(key) end
function Game:keyreleased(key) end
function Game:controlPressed(control) end
function Game:controlReleased(control) end

-- Add states
for k,v in pairs(lib.listFiles("src/states")) do
	require(v)
end
