--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local MenuItem = require "src/libraries/menu/item/item"
local Panel = MenuItem:subclass("Panel")

-- Initialize
function Panel:initialize(x, y, parent)
	MenuItem.initialize(self, x, y, parent)

	-- List of children
	self.children = {}

	-- Layout (vertical/horizontal/grid)
	self.layout = nil

	-- Padding between content
	self.padding = 0

	-- Currently selected child
	self.currentSelected = 0

	-- Show the background
	self.background = true
end

-- Layout & padding
function Panel:setLayout(l, p) self.layout = l if p then self.padding = p end self:refresh() end
function Panel:verticalLayout(p) self.layout = "vertical" if p then self.padding = p end self:refresh() end
function Panel:horizontalLayout(p) self.layout = "horizontal" if p then self.padding = p end self:refresh() end
function Panel:gridLayout(p) self.layout = "grid" if p then self.padding = p end self:refresh() end
function Panel:setPadding(p) self.padding = p self:refresh() end

-- Background
function Panel:setBackground(b) self.background = b end
function Panel:showBackground() self:setBackground(true) end
function Panel:hideBackground() self:setBackground(false) end

-- Refresh content
function Panel:refresh()
	if table.getn(self.children) == 0 or self.layout == nil then return end

	-- Position child elements
	local x = 0
	local y = 0
	if self.layout == "vertical" then
		-- One after the other, top to bottom
		for i,v in ipairs(self.children) do
			v:refresh()
			x = (self.width - v:getWidth()) / 2
			y = y + self.padding
			v:setPosition(x, y)
			y = y + v:getHeight()
		end
		y = y + self.padding
		self.height = y
	elseif self.layout == "horizontal" then
		-- One after the other, left to right
		for i,v in ipairs(self.children) do
			v:refresh()
			y = (self.height - v:getHeight()) / 2
			x = x + self.padding
			v:setPosition(x, y)
			x = x + v:getWidth()
		end
		x = x + self.padding
		self.width = x
	elseif self.layout == "grid" then
		y = y + self.padding
		-- Alternating left and right, top to bottom
		local c = 1
		for i,v in ipairs(self.children) do
			v:refresh()
			if v.colspan then
				x = (self.width - v:getWidth()) / 2
			elseif c % 2 == 1 then
				x = self.padding
			else
				x = self.width - v:getWidth() - self.padding
			end
			v:setPosition(x, y)
			if c % 2 == 0 or v.colspan then
				y = y + v:getHeight() + self.padding
			end
			if v.colspan then
				c = c + 2
			else
				c = c + 1
			end
		end
		self.height = y
	end

	-- Center the location
	self.x = ((love.graphics.getWidth() * game.scale) - self.width) / 2
	self.y = ((love.graphics.getHeight() * game.scale) - self.height) / 2
end

-- Draw
function Panel:draw()
	love.graphics.push()

	if self.background then
		love.graphics.setColor(media.colors.menu.background)
		love.graphics.rectangle("fill", self:getX(),self:getY(),self:getWidth(),self:getHeight())
		love.graphics.setColor(media.colors.menu.border)
		love.graphics.rectangle("line", self:getX()-1,self:getY()-1,self:getWidth()+2,self:getHeight()+2)
	end

	self:drawChildren()

	love.graphics.pop()
end

function Panel:drawChildren()
	love.graphics.push()
	love.graphics.translate(self.x, self.y)
	for i,v in ipairs(self.children) do
		if v.visible then v:draw() end
	end
	love.graphics.pop()
end

-- Update
function Panel:update(dt)
	self:updateChildren(dt)
end

function Panel:updateChildren(dt)
	for i,v in ipairs(self.children) do
		if v.active and v.visible then v:update(dt) end
	end
end

-- Handle mouse presses
function Panel:mousepressed(x, y, button)
	x = x - self.x
	y = y - self.y
	for i,v in ipairs(self.children) do
		if v.active and v.visible then v:mousepressed(x, y, button) end
	end
end

function Panel:mousereleased(x, y, button)
	x = x - self.x
	y = y - self.y
	for i,v in ipairs(self.children) do
		if v.active and v.visible then v:mousereleased(x, y, button) end
	end
end

-- Handle keypresses
function Panel:keypressed(key)
	if table.getn(self.children) == 0 then return end

	-- Navigate
	if key == "down" or key == "right" then
		self:selectNextChild()
	elseif key == "up" or key == "left" then
		self:selectPreviousChild()
	elseif key == "enter" then
		self:action()
	elseif key == "escape" then
		self:triggerEvent("escape", {})
	end

	self:triggerEvent("keypressed", { key = key })

	-- Captue the keypress
	return true
end

function Panel:keyreleased(key)
	for i,v in ipairs(self.children) do
		if v.active and v.visible then v:keyreleased(key) end
	end
end

-- Visibility
function Panel:setVisibility(v)
	MenuItem.setVisibility(self, v)
	if v == false then
		self:selectChild(0)
	end
end

-- Perform action
function Panel:action()
	if self.currentSelected == 0 or table.getn(self.children) == 0 then return end
	self.children[self.currentSelected]:action()
end

-- Select children
function Panel:selectChild(i)
	-- Deselect previous
	if self.currentSelected ~= 0 then
		self.children[self.currentSelected]:deselect()
	end

	self.currentSelected = i

	-- Select new one
	if self.currentSelected ~= 0 then
		self.children[self.currentSelected]:select()
	end
end

function Panel:selectNextChild()
	-- Get next selectable child
	local i = self.currentSelected + 1
	local current = 0
	for i = i,self.currentSelected+table.getn(self.children) do
		if i > table.getn(self.children) then i = i - table.getn(self.children) end
		if self.children[i]:isSelectable() then
			current = i
			break
		end
	end
	self:selectChild(current)
end

function Panel:selectPreviousChild()
	-- Get next selectable child
	local i = self.currentSelected - 1
	if i < 0 then i = 0 end
	local current = 0
	for i = i,self.currentSelected-table.getn(self.children),-1 do
		if i < 1 then i = i + table.getn(self.children) end
		if self.children[i]:isSelectable() then
			current = i
			break
		end
	end
	self:selectChild(current)
end

-- A child button has been hovered
function Panel:buttonHover(button)
	if table.getn(self.children) == 0 then return end

	if self.currentSelected ~= 0 then
		self.children[self.currentSelected]:deselect()
	end

	for i,v in ipairs(self.children) do
		if v.text and v.text == button.text then
			self.currentSelected = i
			v:select()
			return
		end
	end
end

-- Add children
function Panel:addChild(class, x, y)
	class = require("src/libraries/menu/item/" .. class)
	local object = class:new(x, y, self)
	table.insert(self.children, object)
	return object
end

function Panel:addTextButton(text, x, y)
	local button = self:addChild("textbutton", x, y)
	button:setText(text)
	button:setFont(love.graphics.getFont())
	button:addEventListener("hover", function(event, object, params)
		self:buttonHover(object)
	end)
	button:addEventListener("click", function(event, object, params)
		if config.sound then
			love.audio.play(media.audio.click)
		end
	end)
	return button
end

function Panel:addLabel(text, x, y)
	local label = self:addChild("label", x, y)
	label:setText(text)
	return label
end

function Panel:addImage(img, x, y)
	local image = self:addChild("image", x, y)
	image:setImage(img)
	return image
end

function Panel:addImageButton(image, x, y)
	local button = self:addChild("imagebutton", x, y)
	button:setDefaultImage(image)
	button:addEventListener("hover", function(event, object, params)
		self:buttonHover(object)
	end)
	button:addEventListener("click", function(event, object, params)
		if config.sound then
			love.audio.play(media.audio.click)
		end
	end)
	return button
end

function Panel:addScaleButton(image, x, y)
	local button = self:addChild("scalebutton", x, y)
	button:setImage(image)
	button:addEventListener("hover", function(event, object, params)
		self:buttonHover(object)
	end)
	button:addEventListener("click", function(event, object, params)
		if config.sound then
			love.audio.play(media.audio.click)
		end
	end)
	return button
end

function Panel:addCheckbox(x, y)
	local checkbox = self:addChild("checkbox", x, y)
	checkbox:addEventListener("hover", function(event, object, params)
		self:buttonHover(object)
	end)
	checkbox:addEventListener("toggle", function(event, object, params)
		if config.sound then
			love.audio.play(media.audio.click)
		end
	end)
	return checkbox
end

function Panel:addSelection(options, x, y)
	local selection = self:addChild("selection", x, y)
	selection:setFont(love.graphics.getFont())
	selection:setOptions(options)
	selection:addEventListener("change", function(event, object, params)
		if config.sound then
			love.audio.play(media.audio.click)
		end
	end)
	return selection
end

-- Remove children
function Panel:clearChildren()
	self.currentSelected = 0
	self.children = {}
end

return Panel
