--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local GameState = Game:addState("Game")

function GameState:enteredState()
	lib.music:play(media.music.game)
end

function GameState:continuedState()
	if self.paused then
		media.menus.pause:show()
	end
	lib.music:play(media.music.game)
end

function GameState:exitedState()
	media.menus.pause:hide()
	media.menus.scoreboard:hide()
end

function GameState:draw()
	
	-- Draw the background
	local scale = love.graphics.getWidth() / media.images.background:getWidth()
	local y = (media.images.background:getHeight() * scale) - love.graphics.getHeight()
	love.graphics.draw(media.images.background, 0, -y)
	love.graphics.draw(media.images.landscape, 0, -y)

	-- Draw the clouds
	love.graphics.draw(media.images.clouds, self.clouds.x, self.clouds.y)
	
	-- Draw the entities
	love.graphics.push()
	love.graphics.translate(self.level.x, self.level.y)
	
	self.level:draw()
	self.player:draw()
	for i=1,table.getn(self.guards) do
		self.guards[i]:draw()
	end
	
	love.graphics.pop()

	-- Damage screen
	if self.damage > 0 then
		love.graphics.setColorMode("modulate")
		love.graphics.setColor(255, 255, 255, self.damage)
		love.graphics.draw(media.images.damage, 0, 0, 0, 1, 1/self.aspectratio)
		love.graphics.setColorMode("replace")
	end
	
end

function GameState:update(dt)

	-- Pause menu
	if self.paused or self.finishedLevel then
		media.menus.pause:update(dt)
		return
	end
	
	-- Give new instructions to the player
	if self.player.ready then
		
		-- Is the player at the exit?
		if self.player.x == self.level.exit.x and self.player.y == self.level.exit.y then
			if config.sound then
				love.audio.play(media.audio.kungfu)
			end
			self:endLevel()
			return
		end
		
		-- Can player be seen?
		for i=1,table.getn(self.guards) do
			if self.guards[i]:canSee(self.level, self.player.x, self.player.y) then
				if config.sound then
					love.audio.play(media.audio.attack)
				end
				self.damage = 255
				tween(1, self, { damage = 0 } )
				self:restartLevel()
				return
			end
		end
		
		-- Commit instructions
		if self.instructions ~= nil and table.getn(self.instructions) ~= 0 then
			-- Move the player
			self.player:setTarget(self.instructions[1].x, self.instructions[1].y)

			-- Move the guards
			for i=1,table.getn(self.guards) do
				self.guards[i]:move(self.level)
			end
		
			table.remove(self.instructions, 1)
			self.moves = self.moves + 1
		end
	end
	
	-- Update the entities
	self.level:update(dt)
	self.player:update(dt)
	for i=1,table.getn(self.guards) do
		self.guards[i]:update(dt)
	end

end

function GameState:mousepressed(x, y, button)

	-- Paused
	if self.paused or self.finishedLevel then return end

	-- Get the position in the level
	if (button == 'l') and table.getn(self.instructions) == 0 then
		local pos = self.level:reverseTranslatePosition(x,y)
		if pos and self.level:isOpen(pos.x, pos.y) then
			
			-- Build instructions for the player
			self.instructions = lib.astar(self.level.map, { x = self.player.x, y = self.player.y }, pos)
			if self.instructions == nil then self.instructions = {} end -- prevents errors elsewhere
			
		end
	end
end

function GameState:controlPressed(control)

	-- Pause menu
	if self.paused or self.finishedLevel then
		menu:keypressed(control)
		return
	elseif control == 'escape' then
		self.paused = true
		media.menus.pause:show()
		return
	end

	-- Player movement
	if table.getn(self.instructions) == 0 then
		if (control == 'up') and self.level:isOpen(self.player.x, self.player.y - 1) then
			table.insert(self.instructions, { x = 0, y = -1 })
		elseif (control == 'right') and self.level:isOpen(self.player.x + 1, self.player.y) then
			table.insert(self.instructions, { x = 1, y = 0 })
		elseif (control == 'down') and self.level:isOpen(self.player.x, self.player.y + 1) then
			table.insert(self.instructions, { x = 0, y = 1 })
		elseif (control == 'left') and self.level:isOpen(self.player.x - 1, self.player.y) then
			table.insert(self.instructions, { x = -1, y = 0 })
		end
	end

end
